Module: Miscellany
- Defined in:
- lib/miscellany/active_record/custom_preloaders.rb,
lib/miscellany.rb,
lib/miscellany/version.rb,
lib/miscellany/sort_lang.rb,
lib/miscellany/batch_processor.rb,
lib/miscellany/local_lru_cache.rb,
lib/miscellany/param_validator.rb,
lib/miscellany/batching_csv_processor.rb,
lib/miscellany/jbuilder_partial_block.rb,
lib/miscellany/controller/json_uploads.rb,
lib/miscellany/controller/sliced_response.rb,
lib/miscellany/active_record/batch_matcher.rb,
lib/miscellany/active_record/complex_query.rb,
lib/miscellany/active_record/goldiload_value.rb,
lib/miscellany/active_record/computed_columns.rb,
lib/miscellany/controller/http_error_handling.rb,
lib/miscellany/active_record/arbitrary_prefetch.rb,
lib/miscellany/active_record/batched_destruction.rb
Overview
Add support for creating arbitrary associations when using ActiveRecord
Adds a prefetch method to ActiveRecord Queries.
This method accepts a Hash. The keys of the Hash represent how the Association will be made available.
The values of the Hash may be an array of [Symbol, Relation or Proc], a Proc, or another (filtered) Relation.
Objects are queried from an existing Association on the model. This Association is detemrined
by either the Symbol when an array is passed, the association named in a Proc,
or by finding an Assoication for the passed Relation's model
Defined Under Namespace
Modules: ArbitraryPrefetch, BatchedDestruction, ComputedColumns, CustomPreloaders, Extensions, GoldiloadValue, HttpErrorHandling, JsonUploads, SlicedResponse, SortLang Classes: BatchMatcher, BatchProcessor, BatchingCsvProcessor, ComplexQuery, Engine, LocalLruCache, ParamValidator
Constant Summary collapse
- VERSION =
"0.2.0".freeze